# -*- coding: utf-8 -*-
from Acquisition import aq_base
from Acquisition import aq_inner
from plone.app.content.interfaces import INameFromTitle
from plone.i18n.normalizer import FILENAME_REGEX
from plone.i18n.normalizer.interfaces import IURLNormalizer
from plone.i18n.normalizer.interfaces import IUserPreferredURLNormalizer
from zExceptions import BadRequest
from zope.component import getUtility
from zope.container.interfaces import INameChooser
from zope.interface import implementer

import six
import time


ATTEMPTS = 100


@implementer(INameChooser)
class NormalizingNameChooser(object):
    """A name chooser for a Zope object manager.

    If the object is adaptable to or provides INameFromTitle, use the
    title to generate a name.
    """

    def __init__(self, context):
        self.context = context

    def checkName(self, name, obj):
        return not self._getCheckId(obj)(name, required=1)

    def chooseName(self, name, obj):
        container = aq_inner(self.context)
        if not name:
            nameFromTitle = INameFromTitle(obj, None)
            if nameFromTitle is not None:
                name = nameFromTitle.title
            if not name:
                name = getattr(aq_base(obj), 'id', None)
            if not name:
                name = getattr(aq_base(obj), 'portal_type', None)
            if not name:
                name = obj.__class__.__name__

        if not isinstance(name, six.text_type):
            name = six.text_type(name, 'utf-8')
            #name = name.encode('utf-8')

        request = getattr(obj.__of__(container), 'REQUEST', None)
        if request is not None:
            name = IUserPreferredURLNormalizer(request).normalize(name)
        else:
            name = getUtility(IURLNormalizer).normalize(name)

        return self._findUniqueName(name, obj)

    def _findUniqueName(self, name, obj):
        """Find a unique name in the parent folder, based on the given id, by
        appending -n, where n is a number greater than 1, or just the id if
        it's ok.
        """
        check_id = self._getCheckId(obj)

        if not check_id(name, required=1):
            return name

        ext = ''
        m = FILENAME_REGEX.match(name)
        if m is not None:
            name = m.groups()[0]
            ext = '.' + m.groups()[1]

        idx = 1
        while idx <= ATTEMPTS:
            new_name = "%s-%d%s" % (name, idx, ext)
            if not check_id(new_name, required=1):
                return new_name
            idx += 1

        # give one last attempt using the current date time before giving up
        new_name = "%s-%s%s" % (name, time.time(), ext)
        if not check_id(new_name, required=1):
            return new_name

        raise ValueError(
            "Cannot find a unique name based on %s after %d attemps." % (
                name,
                ATTEMPTS,
            )
        )

    def _getCheckId(self, obj):
        """Return a function that can act as the check_id script.
        """
        parent = aq_inner(self.context)
        # Check for a method or a skin script, like
        # Products/CMFPlone/skins/plone_scripts/check_id.py until Plone 5.1.
        _check_id = getattr(obj, 'check_id', None)

        def do_Plone_check(newid, required):
            if _check_id is not None:
                return _check_id(
                    newid,
                    required=required,
                    contained_by=parent
                )

            # Function in CMFPlone added in 5.1.4/5,
            # which replaces the skin script that will be removed in 5.2.
            try:
                from Products.CMFPlone.utils import check_id
                return check_id(
                    obj, newid, required=required, contained_by=parent)
            except ImportError:
                pass

            # fallback to OFS
            try:
                parent._checkId(newid)
            except BadRequest:
                return True

        return do_Plone_check



# -*- coding: utf-8 -*-
from plone.app.content.browser.contents import ContentsBaseAction
from Products.CMFPlone import PloneMessageFactory as _


class SetDefaultPageActionView(ContentsBaseAction):
    success_msg = _(u'Default page set successfully')
    failure_msg = _(u'Failed to set default page')

    def __call__(self):
        cid = self.request.form.get('id')
        self.errors = []

        if cid not in self.context.objectIds():
            self.errors.append(
                _(u'There is no object with short name '
                  u'${name} in this folder.',
                  mapping={u'name': cid}))
        else:
            self.context.setDefaultPage(cid)
        return self.message()



# -*- coding: utf-8 -*-
from Acquisition import aq_get
from plone.app.content.testing import PLONE_APP_CONTENT_INTEGRATION_TESTING

import unittest


class AddingTests(unittest.TestCase):

    layer = PLONE_APP_CONTENT_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_adding_acquisition(self):
        adding = self.portal.unrestrictedTraverse('+')
        # Check explicit Acquisition
        template = aq_get(adding, 'portal_skins')
        self.assertTrue(template)
        # Check implicit Acquisition, unfortunately the CMF skins machinery
        # depends on this
        template = getattr(adding, 'portal_skins')
        self.assertTrue(template)
        # Check traversal
        self.assertTrue(self.portal.unrestrictedTraverse('+/main_template'))



# -*- coding: utf-8 -*-
from plone.app.content.testing import HAS_AT
from plone.app.content.testing import PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.testing.z2 import Browser
from plone.locking.interfaces import ILockable
from zExceptions import Unauthorized
from z3c.form.interfaces import IFormLayer
from zope.component import getMultiAdapter
from zope.interface import alsoProvides

import transaction
import unittest

if HAS_AT:
    from plone.app.content.testing import PLONE_APP_CONTENT_AT_FUNCTIONAL_TESTING

class ActionsDXTestCase(unittest.TestCase):

    layer = PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        self.portal.acl_users.userFolderAddUser(
            'editor', 'secret', ['Editor'], [])

        # For z3c.forms request must provide IFormLayer
        alsoProvides(self.request, IFormLayer)

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory(
            type_name='Folder', id='f1', title=u'A Tést Folder')

        transaction.commit()
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization', 'Basic {0}:{1}'.format(TEST_USER_NAME, 'secret'))

    def tearDown(self):
        if 'f1' in self.portal.objectIds():
            self.portal.manage_delObjects(ids='f1')
            transaction.commit()

    def test_delete_confirmation(self):
        folder = self.portal['f1']

        form = getMultiAdapter(
            (folder, self.request), name='delete_confirmation')
        form.update()

        cancel = form.buttons['Cancel']
        form.handlers.getHandler(cancel)(form, form)

        self.assertFalse(form.is_locked)

    def test_delete_confirmation_if_locked(self):
        folder = self.portal['f1']
        lockable = ILockable.providedBy(folder)

        form = getMultiAdapter(
            (folder, self.request), name='delete_confirmation')
        form.update()

        self.assertFalse(form.is_locked)

        if lockable:
            lockable.lock()

        form = getMultiAdapter(
            (folder, self.request), name='delete_confirmation')
        form.update()

        self.assertFalse(form.is_locked)

        # After switching the user it should not be possible to delete the
        # object. Of course this is only possible if our context provides
        # ILockable interface.
        if lockable:
            logout()
            login(self.portal, 'editor')

            form = getMultiAdapter(
                (folder, self.request), name='delete_confirmation')
            form.update()
            self.assertTrue(form.is_locked)

            logout()
            login(self.portal, TEST_USER_NAME)

            ILockable(folder).unlock()

    def test_delete_confirmation_cancel(self):
        folder = self.portal['f1']

        self.browser.open(folder.absolute_url() + '/delete_confirmation')
        self.browser.getControl(name='form.buttons.Cancel').click()
        context_state = getMultiAdapter(
            (folder, self.request),
            name='plone_context_state'
        )
        self.assertEqual(self.browser.url, context_state.view_url())

    def prepare_for_acquisition_tests(self):
        """create content and an alternate authenticated browser session

        creates the following content structure:

        |-- f1
        |   |-- test
        |-- test
        """
        # create a page at the root and one nested with the same id.
        p1 = self.portal.invokeFactory(
            type_name='Document', id='test', title='Test Page at Root')
        folder_1 = self.portal['f1']
        p2 = folder_1.invokeFactory(
            type_name='Document', id='test', title='Test Page in Folder')
        contained_test_page = folder_1[p2]

        transaction.commit()

        # create an alternate browser also logged in with manager
        browser_2 = Browser(self.layer['app'])
        browser_2.handleErrors = False
        browser_2.addHeader(
            'Authorization', 'Basic {0}:{1}'.format(TEST_USER_NAME, 'secret'))

        # return the id of the root page, the nested page itself, and the
        # alternate browser
        return p1, contained_test_page, browser_2

    def test_delete_wrong_object_by_acquisition_with_action(self):
        """exposes delete-by-acquisition bug using the delete action

        see https://github.com/plone/Products.CMFPlone/issues/383
        """
        p1_id, page_2, browser_2 = self.prepare_for_acquisition_tests()

        # open two different browsers to the 'delete confirmation' view
        delete_url = page_2.absolute_url() + '/delete_confirmation'
        self.browser.open(delete_url)
        browser_2.open(delete_url)
        self.assertTrue(p1_id in self.portal)
        for browser in [self.browser, browser_2]:
            browser.getControl(name='form.buttons.Delete').click()

        # the nested folder should be gone, but the one at the root should
        # remain.
        self.assertFalse(page_2.id in self.portal['f1'])
        self.assertTrue(p1_id in self.portal)

    def test_rename_form(self):
        logout()
        folder = self.portal['f1']

        # We need zope2.CopyOrMove permission to rename content
        self.browser.open(folder.absolute_url() + '/folder_rename')
        self.browser.getControl(name='form.widgets.new_id').value = 'f2'
        self.browser.getControl(name='form.widgets.new_title').value = 'F2'
        self.browser.getControl(name='form.buttons.Rename').click()
        self.assertEqual(folder.getId(), 'f2')
        self.assertEqual(folder.Title(), 'F2')
        self.assertEqual(self.browser.url, folder.absolute_url())

        login(self.portal, TEST_USER_NAME)
        self.portal.manage_delObjects(ids='f2')
        transaction.commit()

    def test_rename_form_with_view_action(self):
        # can't be bothered to register blobs, instead we add documents to
        # typesUseViewActionInListings
        registry = self.portal.portal_registry
        registry['plone.types_use_view_action_in_listings'] = [u'Image',
                                                               u'File',
                                                               u'Document']

        folder = self.portal['f1']
        folder.invokeFactory('Document', 'document1')
        document1 = folder['document1']
        transaction.commit()
        logout()

        # We need zope2.CopyOrMove permission to rename content
        self.browser.open(document1.absolute_url() + '/object_rename')
        self.browser.getControl(name='form.widgets.new_id').value = 'f2'
        self.browser.getControl(name='form.widgets.new_title').value = 'F2'
        self.browser.getControl(name='form.buttons.Rename').click()
        self.assertEqual(document1.getId(), 'f2')
        self.assertEqual(document1.Title(), 'F2')
        self.assertEqual(self.browser.url, document1.absolute_url() + '/view')

        login(self.portal, TEST_USER_NAME)
        self.portal.manage_delObjects(ids='f1')
        transaction.commit()

    def test_create_safe_id_on_renaming(self):
        logout()
        folder = self.portal['f1']

        # We need zope2.CopyOrMove permission to rename content
        self.browser.open(folder.absolute_url() + '/folder_rename')
        self.browser.getControl(name='form.widgets.new_id').value = ' ? f4 4 '
        self.browser.getControl(name='form.widgets.new_title').value = ' F2 '
        self.browser.getControl(name='form.buttons.Rename').click()
        self.assertEqual(folder.getId(), 'f4-4')
        self.assertEqual(folder.Title(), 'F2')
        self.assertEqual(self.browser.url, folder.absolute_url())

        login(self.portal, TEST_USER_NAME)
        self.portal.manage_delObjects(ids='f4-4')
        transaction.commit()

    def test_default_page_updated_on_rename(self):
        login(self.portal, TEST_USER_NAME)
        folder = self.portal['f1']
        folder.invokeFactory(type_name='Document', id='d1', title='A Doc')
        doc = folder['d1']
        folder.setDefaultPage('d1')
        transaction.commit()
        self.assertEqual(folder.getDefaultPage(), 'd1')

        # We need zope2.CopyOrMove permission to rename content
        self.browser.open(doc.absolute_url() + '/object_rename')
        self.browser.getControl(name='form.widgets.new_id').value = ' ?renamed'
        self.browser.getControl(name='form.widgets.new_title').value = 'Doc'
        self.browser.getControl(name='form.buttons.Rename').click()
        self.assertEqual(folder.getFolderContents()[0].id, 'renamed')
        self.assertEqual(folder.getDefaultPage(), 'renamed')

    def test_rename_form_cancel(self):
        folder = self.portal['f1']

        _id = folder.getId()
        _title = folder.Title()

        self.browser.open(folder.absolute_url() + '/folder_rename')
        self.browser.getControl(name='form.buttons.Cancel').click()
        transaction.commit()

        self.assertEqual(self.browser.url, folder.absolute_url())
        self.assertEqual(folder.getId(), _id)
        self.assertEqual(folder.Title(), _title)

    def test_rename_form_cancel_with_view_action(self):
        # can't be bothered to register blobs, instead we add documents to
        # typesUseViewActionInListings
        registry = self.portal.portal_registry
        registry['plone.types_use_view_action_in_listings'] = [u'Image',
                                                               u'File',
                                                               u'Document']
        folder = self.portal['f1']
        folder.invokeFactory('Document', 'document1')
        document1 = folder['document1']
        transaction.commit()

        _id = document1.getId()
        _title = document1.Title()

        self.browser.open(document1.absolute_url() + '/object_rename')
        self.browser.getControl(name='form.buttons.Cancel').click()
        transaction.commit()

        self.assertEqual(self.browser.url, document1.absolute_url() + '/view')
        self.assertEqual(document1.getId(), _id)
        self.assertEqual(document1.Title(), _title)

    def _get_token(self, context):
        authenticator = getMultiAdapter(
            (context, self.request), name='authenticator')

        return authenticator.token()

    def test_object_cut_view(self):
        folder = self.portal['f1']

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized,
            self.browser.open,
            '{0:s}/object_cut'.format(folder.absolute_url())
        )

        # We need to have Copy or Move permission to cut an object
        self.browser.open('{0:s}/object_cut?_authenticator={1:s}'.format(
            folder.absolute_url(), self._get_token(folder)))

        self.assertIn('__cp', self.browser.cookies)
        self.assertIn(
            '{0:s} cut.'.format(folder.Title()), self.browser.contents)

    def test_object_cut_view_with_view_action(self):
        # can't be bothered to register blobs, instead we add documents to
        # typesUseViewActionInListings
        registry = self.portal.portal_registry
        registry['plone.types_use_view_action_in_listings'] = [u'Image',
                                                               u'File',
                                                               u'Document']
        folder = self.portal['f1']
        folder.invokeFactory('Document', 'document1')
        document1 = folder['document1']
        transaction.commit()

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized,
            self.browser.open,
            '{0:s}/object_cut'.format(document1.absolute_url())
        )

        # We need to have Copy or Move permission to cut an object
        self.browser.open('{0:s}/object_cut?_authenticator={1:s}'.format(
            document1.absolute_url(), self._get_token(document1)))

        self.assertIn('__cp', self.browser.cookies)
        self.assertIn(
            '{0:s} cut.'.format(document1.Title()), self.browser.contents)
        self.assertEqual(document1.absolute_url() + '/view', self.browser.url)

    def test_object_copy_view(self):
        folder = self.portal['f1']

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized,
            self.browser.open,
            '{0:s}/object_copy'.format(folder.absolute_url())
        )

        self.browser.open('{0:s}/object_copy?_authenticator={1:s}'.format(
            folder.absolute_url(), self._get_token(folder)))

        self.assertIn('__cp', self.browser.cookies)
        self.assertIn(
            '{0:s} copied.'.format(folder.Title()), self.browser.contents)

    def test_object_copy_with_view_action(self):
        # can't be bothered to register blobs, instead we add documents to
        # typesUseViewActionInListings
        registry = self.portal.portal_registry
        registry['plone.types_use_view_action_in_listings'] = [u'Image',
                                                               u'File',
                                                               u'Document']

        folder = self.portal['f1']
        folder.invokeFactory('Document', 'document1')
        document1 = folder['document1']
        transaction.commit()

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized,
            self.browser.open,
            '{0:s}/object_copy'.format(document1.absolute_url())
        )

        self.browser.open('{0:s}/object_copy?_authenticator={1:s}'.format(
            document1.absolute_url(), self._get_token(document1)))

        self.assertIn('__cp', self.browser.cookies)
        self.assertIn(
            '{0:s} copied.'.format(document1.Title()), self.browser.contents)
        self.assertEqual(document1.absolute_url() + '/view', self.browser.url)

    def test_object_cut_and_paste(self):
        folder = self.portal['f1']
        self.portal.invokeFactory(type_name='Document', id='d1', title='A Doc')
        doc = self.portal['d1']
        transaction.commit()

        self.browser.open('{0:s}/object_cut?_authenticator={1:s}'.format(
            doc.absolute_url(), self._get_token(doc)))

        self.assertIn('__cp', self.browser.cookies)
        self.assertIn('d1', self.portal.objectIds())
        self.assertIn('f1', self.portal.objectIds())

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized,
            self.browser.open,
            '{0:s}/object_paste'.format(folder.absolute_url())
        )

        self.browser.open('{0:s}/object_paste?_authenticator={1:s}'.format(
            folder.absolute_url(), self._get_token(doc)))

        self.assertIn('__cp', self.browser.cookies)
        transaction.commit()

        self.assertNotIn('d1', self.portal.objectIds())
        self.assertIn('d1', folder.objectIds())
        self.assertIn('Item(s) pasted.', self.browser.contents)

    def test_object_copy_and_paste(self):
        folder = self.portal['f1']
        folder.invokeFactory(type_name='Document', id='d1', title='A Doc')
        doc = folder['d1']
        transaction.commit()

        self.browser.open('{0:s}/object_copy?_authenticator={1:s}'.format(
            doc.absolute_url(), self._get_token(doc)))

        self.assertIn('__cp', self.browser.cookies)

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized,
            self.browser.open,
            '{0:s}/object_paste'.format(folder.absolute_url())
        )

        self.browser.open('{0:s}/object_paste?_authenticator={1:s}'.format(
            folder.absolute_url(), self._get_token(folder)))
        transaction.commit()

        self.assertIn('f1', self.portal.objectIds())
        self.assertIn('d1', folder.objectIds())
        self.assertIn('copy_of_d1', folder.objectIds())
        self.assertIn('Item(s) pasted.', self.browser.contents)

    def test_object_copy_and_paste_multiple_times(self):
        folder = self.portal['f1']
        folder.invokeFactory(type_name='Document', id='d1', title='A Doc')
        doc = folder['d1']
        transaction.commit()

        self.browser.open('{0:s}/object_copy?_authenticator={1:s}'.format(
            doc.absolute_url(), self._get_token(doc)))

        self.assertIn('__cp', self.browser.cookies)
        self.browser.open('{0:s}/object_paste?_authenticator={1:s}'.format(
            folder.absolute_url(), self._get_token(folder)))
        self.browser.open('{0:s}/object_paste?_authenticator={1:s}'.format(
            folder.absolute_url(), self._get_token(folder)))

        # Cookie should persist, because you can paste the item multiple times
        self.assertIn('__cp', self.browser.cookies)
        self.assertIn('f1', self.portal.objectIds())
        self.assertIn('d1', folder.objectIds())
        self.assertIn('copy_of_d1', folder.objectIds())
        self.assertIn('copy2_of_d1', folder.objectIds())
        self.assertIn('Item(s) pasted.', self.browser.contents)


if HAS_AT:
    class ActionsATTestCase(ActionsDXTestCase):

        layer = PLONE_APP_CONTENT_AT_FUNCTIONAL_TESTING



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from Products.CMFCore.Expression import createExprContext
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from plone.app.content.browser.interfaces import IFolderContentsView
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.memoize.instance import memoize
from plone.memoize.request import memoize_diy_request
from plone.protect.authenticator import createToken
from six.moves.urllib.parse import quote_plus
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.i18n import translate
from zope.publisher.browser import BrowserView


@memoize_diy_request(arg=0)
def _allowedTypes(request, context):
    return context.allowedContentTypes()


class FolderFactoriesView(BrowserView):
    """The folder_factories view - show addable types
    """

    def __call__(self):
        if 'form.button.Add' in self.request.form:
            urltool = getToolByName(self.context, 'portal_url')
            url = self.request.form.get('url')
            if not urltool.isURLInPortal(url):
                url = self.context.absolute_url()
            self.request.response.redirect(url)
            return ''
        else:
            return self.index()

    def can_constrain_types(self):
        aspect = ISelectableConstrainTypes(self.add_context(), None)
        return aspect.canSetConstrainTypes() if aspect else False

    @memoize
    def add_context(self):
        context = self.context
        context_state = getMultiAdapter((context, self.request),
                                        name='plone_context_state')
        context = aq_inner(context)
        try:
            published = self.request.PUBLISHED
        except AttributeError:
            published = context
        if context_state.is_structural_folder():
            if context_state.is_default_page():
                is_folder_contents_view = \
                    IFolderContentsView.providedBy(published)
                if is_folder_contents_view or self == published:
                    # on the folder_contents view and factories view,
                    # show the actual context object's addable types
                    return context
                else:
                    return aq_parent(context)
            else:
                return context
        else:
            return aq_parent(context)
    # NOTE: This is also used by plone.app.contentmenu.menu.FactoriesMenu.
    # The return value is somewhat dictated by the menu infrastructure, so
    # be careful if you change it

    def addable_types(self, include=None):
        """Return menu item entries in a TAL-friendly form.

        Pass a list of type ids to 'include' to explicitly allow a list of
        types.
        """

        context = aq_inner(self.context)
        request = self.request

        results = []

        idnormalizer = queryUtility(IIDNormalizer)
        portal_state = getMultiAdapter((context, request),
                                       name='plone_portal_state')

        addContext = self.add_context()
        baseUrl = addContext.absolute_url()
        token = createToken()

        allowedTypes = _allowedTypes(request, addContext)

        types_tool = getToolByName(context, 'portal_types')

        # Note: we don't check 'allowed' or 'available' here, because these are
        # slow. We assume the 'allowedTypes' list has already performed the
        # necessary calculations
        actions = types_tool.listActionInfos(
            object=addContext,
            check_permissions=False,
            check_condition=False,
            category='folder/add',
        )
        addActionsById = dict([(a['id'], a) for a in actions])

        expr_context = createExprContext(
            aq_parent(addContext), portal_state.portal(), addContext)
        for t in allowedTypes:
            typeId = t.getId()
            if include is None or typeId in include:
                cssId = idnormalizer.normalize(typeId)
                cssClass = 'contenttype-%s' % cssId

                url = None
                addAction = addActionsById.get(typeId, None)
                if addAction is not None:
                    url = addAction['url']

                if not url:
                    url = '%s/createObject?type_name=%s&_authenticator=%s' % (
                        baseUrl, quote_plus(typeId), token)

                icon = t.getIconExprObject()
                if icon:
                    icon = icon(expr_context)

                results.append({
                    'id': typeId,
                    'title': t.Title(),
                    'description': t.Description(),
                    'action': url,
                    'selected': False,
                    'icon': icon,
                    'extra': {
                        'id': cssId,
                        'separator': None,
                        'class': cssClass},
                    'submenu': None,
                })

        # Sort the addable content types based on their translated title
        results = [(translate(ctype['title'], context=request), ctype)
                   for ctype in results]
        results.sort()
        results = [ctype[-1] for ctype in results]

        return results



# -*- coding: utf-8 -*-
from OFS.CopySupport import _cb_encode
from OFS.CopySupport import cookie_path
from OFS.Moniker import Moniker
from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.interfaces import IStructureAction
from Products.CMFPlone import PloneMessageFactory as _
from zope.i18n import translate
from zope.interface import implementer


@implementer(IStructureAction)
class CopyAction(object):

    order = 2

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_options(self):
        return {
            'tooltip': translate(_('Copy'), context=self.request),
            'id': 'copy',
            'icon': 'duplicate',
            'url': self.context.absolute_url() + '/@@fc-copy'
        }


class CopyActionView(ContentsBaseAction):
    success_msg = _('Successfully copied items')
    failure_msg = _('Failed to copy items')

    def action(self, obj):
        self.oblist.append(obj)

    def finish(self):
        oblist = []
        for ob in self.oblist:
            if not ob.cb_isCopyable():
                self.errors.append(_(u'${title} cannot be copied.',
                                     mapping={u'title': self.objectTitle(ob)}))
                continue
            m = Moniker(ob)
            oblist.append(m.dump())
        cp = (0, oblist)
        cp = _cb_encode(cp)
        resp = self.request.response
        resp.setCookie('__cp', cp, path='%s' % cookie_path(self.request))
        self.request['__cp'] = cp

    def __call__(self):
        self.oblist = []
        return super(CopyActionView, self).__call__(keep_selection_order=True)



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from plone.app.content.browser.tableview import Table
from plone.app.content.browser.tableview import TableBrowserView
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import human_readable_size
from Products.CMFPlone.utils import isExpired
from Products.CMFPlone.utils import safe_unicode
from six.moves import map
from six.moves.urllib.parse import quote_plus
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.i18n import translate
from zope.publisher.browser import BrowserView


class FullReviewListView(BrowserView):

    def revlist(self):
        portal_membership = getToolByName(self.context, 'portal_membership')
        portal_workflow = getToolByName(self.context, 'portal_workflow')
        if portal_membership.isAnonymousUser():
            return []

        return portal_workflow.getWorklistsResults()

    def url(self):
        return self.context.absolute_url() + '/full_review_list'

    def review_table(self):
        table = ReviewListTable(self.context, self.request)
        return table.render()


class ReviewListTable(object):
    """
    The review list table renders the table and its actions.
    """

    def __init__(self, context, request, **kwargs):
        self.context = context
        self.request = request

        url = self.context.absolute_url()
        view_url = url + '/full_review_list'
        self.table = Table(request, url, view_url, self.items,
                           buttons=self.buttons)

    def render(self):
        return self.table.render()

    @property
    def items(self):
        plone_utils = getToolByName(self.context, 'plone_utils')
        portal_url = getToolByName(self.context, 'portal_url')
        plone_view = getMultiAdapter((self.context, self.request),
                                     name=u'plone')
        plone_layout = getMultiAdapter((self.context, self.request),
                                       name=u'plone_layout')
        portal_workflow = getToolByName(self.context, 'portal_workflow')
        portal_types = getToolByName(self.context, 'portal_types')
        portal_membership = getToolByName(self.context, 'portal_membership')

        registry = getUtility(IRegistry)
        use_view_action = registry.get(
            'plone.types_use_view_action_in_listings', ())

        browser_default = plone_utils.browserDefault(self.context)

        results = list()
        if portal_membership.isAnonymousUser():
            worklist = []
        else:
            worklist = portal_workflow.getWorklistsResults()

        for i, obj in enumerate(worklist):
            if i % 2 == 0:
                table_row_class = "even"
            else:
                table_row_class = "odd"

            url = obj.absolute_url()
            path = '/'.join(obj.getPhysicalPath())
            type_class = 'contenttype-' + plone_utils.normalizeString(
                obj.portal_type)

            review_state = portal_workflow.getInfoFor(obj, 'review_state', '')

            state_class = 'state-' + plone_utils.normalizeString(review_state)
            relative_url = portal_url.getRelativeContentURL(obj)

            type_title_msgid = portal_types[obj.portal_type].Title()
            url_href_title = u'%s: %s' % (translate(type_title_msgid,
                                                    context=self.request),
                                          safe_unicode(obj.Description()))
            getMember = getToolByName(obj, 'portal_membership').getMemberById
            creator_id = obj.Creator()
            creator = getMember(creator_id)
            if creator:
                creator_name = creator.getProperty('fullname', '') or creator_id
            else:
                creator_name = creator_id
            modified = ''.join(map(safe_unicode, [
                creator_name, ' - ',
                plone_view.toLocalizedTime(obj.ModificationDate(),
                                           long_format=1)]))
            is_structural_folder = obj.restrictedTraverse(
                '@@plone').isStructuralFolder()

            if obj.portal_type in use_view_action:
                view_url = url + '/view'
            elif is_structural_folder:
                view_url = url + "/folder_contents"
            else:
                view_url = url

            is_browser_default = len(browser_default[1]) == 1 and (
                obj.id == browser_default[1][0])

            results.append(dict(
                url=url,
                url_href_title=url_href_title,
                id=obj.getId(),
                quoted_id=quote_plus(obj.getId()),
                path=path,
                title_or_id=obj.pretty_title_or_id(),
                description=obj.Description(),
                obj_type=obj.Type,
                size=human_readable_size(obj.get_size()),
                modified=modified,
                type_class=type_class,
                wf_state=review_state,
                state_title=portal_workflow.getTitleForStateOnType(
                    review_state, obj.portal_type),
                state_class=state_class,
                is_browser_default=is_browser_default,
                folderish=is_structural_folder,
                relative_url=relative_url,
                view_url=view_url,
                table_row_class=table_row_class,
                is_expired=isExpired(obj)
            ))
        return results

    @property
    def show_sort_column(self):
        return False

    def buttons(self):
        buttons = []
        portal_actions = getToolByName(self.context, 'portal_actions')
        button_actions = portal_actions.listActionInfos(
            object=aq_inner(self.context), categories=('folder_buttons', ))

        # Do not show buttons if there is no data, unless there is data to be
        # pasted
        if False:  # not len(self.batch):
            if self.context.cb_dataValid():
                for button in button_actions:
                    if button['id'] == 'paste':
                        return [self.setbuttonclass(button)]
            else:
                return []

        for button in button_actions:
            # Make proper classes for our buttons
            if button['id'] != 'paste' or self.context.cb_dataValid():
                buttons.append(self.setbuttonclass(button))
        return buttons

    def setbuttonclass(self, button):
        if button['id'] == 'paste':
            button['cssclass'] = 'standalone'
        else:
            button['cssclass'] = 'context'
        return button


class ReviewListBrowserView(TableBrowserView):
    table = ReviewListTable



# -*- coding: utf-8 -*-
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFPlone.DublinCore import DefaultDublinCoreImpl
from plone.app.content.interfaces import IReindexOnModify
from zope.container.contained import Contained
from zope.interface import implementer


@implementer(IReindexOnModify)
class Item(PortalContent, DefaultDublinCoreImpl, Contained):
    """A non-containerish, CMFish item
    """

    def __init__(self, id=None, **kwargs):
        DefaultDublinCoreImpl.__init__(self, **kwargs)
        if id is not None:
            self.id = id



# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone.app.content.testing import HAS_AT
from plone.app.content.testing import PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING
from plone.app.content.testing import PLONE_APP_CONTENT_DX_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.dexterity.fti import DexterityFTI
from plone.locking.interfaces import IRefreshableLockable
from plone.protect.authenticator import createToken
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from six.moves.urllib.parse import urlparse
from Testing.makerequest import makerequest
from transaction import commit
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.interface import alsoProvides
from zope.publisher.browser import TestRequest

import json
import unittest

if HAS_AT:
    from plone.app.content.testing import PLONE_APP_CONTENT_AT_INTEGRATION_TESTING


class BaseTest(unittest.TestCase):

    def setUp(self):
        self.portal = self.layer['portal']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.invokeFactory('Document', id="page", title="page")
        self.portal.page.reindexObject()

        self.request = TestRequest(
            environ={
                'HTTP_ACCEPT_LANGUAGE': 'en',
                'REQUEST_METHOD': 'POST'
            },
            form={
                'selection': '["' + IUUID(self.portal.page) + '"]',
                '_authenticator': createToken(),
                'folder': '/'
            }
        )
        self.request.REQUEST_METHOD = 'POST'
        # Mock physicalPathFromURL
        # NOTE: won't return the right path in virtual hosting environments
        self.request.physicalPathFromURL = lambda url: urlparse(url).path.split('/')  # noqa
        alsoProvides(self.request, IAttributeAnnotatable)
        self.userList = 'one,two'


class DXBaseTest(BaseTest):

    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        portal_types = getToolByName(self.portal, "portal_types")
        if 'Document' not in portal_types.objectIds():
            fti = DexterityFTI('Document')
            portal_types._setObject('Document', fti)
        super(DXBaseTest, self).setUp()


class PropertiesDXTest(DXBaseTest):

    def testEffective(self):
        from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
        self.request.form['effectiveDate'] = '1999/01/01 09:00'
        view = PropertiesActionView(self.portal.page, self.request)
        view()
        self.assertEqual(self.portal.page.effective_date,
                          DateTime('1999/01/01 09:00'))

    def testExpires(self):
        from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
        self.request.form['expirationDate'] = '1999/01/01 09:00'
        view = PropertiesActionView(self.portal.page, self.request)
        view()
        self.assertEqual(self.portal.page.expiration_date,
                          DateTime('1999/01/01 09:00'))

    def testSetDexterityExcludeFromNav(self):
        from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
        self.request.form['exclude-from-nav'] = 'yes'
        view = PropertiesActionView(self.portal.page, self.request)
        view()
        self.assertEqual(self.portal.page.exclude_from_nav, True)

    def testRights(self):
        from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
        self.request.form['copyright'] = 'foobar'
        view = PropertiesActionView(self.portal.page, self.request)
        view()
        self.assertEqual(self.portal.page.rights, 'foobar')

    def testContributors(self):
        from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
        self.request.form['contributors'] = self.userList
        view = PropertiesActionView(self.portal.page, self.request)
        view()
        self.assertEqual(self.portal.page.contributors, ('one', 'two'))

    def testCreators(self):
        from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
        self.request.form['creators'] = self.userList
        view = PropertiesActionView(self.portal.page, self.request)
        view()
        self.assertEqual(
            self.portal.page.creators,
            ('one', 'two')
        )

if HAS_AT:
    class PropertiesArchetypesTest(BaseTest):
        layer = PLONE_APP_CONTENT_AT_INTEGRATION_TESTING

        def testExcludeFromNav(self):
            from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
            self.request.form['exclude-from-nav'] = 'yes'
            view = PropertiesActionView(self.portal.page, self.request)
            view()
            self.assertEqual(self.portal.page.getExcludeFromNav(), True)

        def testEffective(self):
            from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
            self.request.form['effectiveDate'] = '1999/01/01 09:00'
            view = PropertiesActionView(self.portal.page, self.request)
            view()
            self.assertEqual(
                DateTime(self.portal.page.EffectiveDate()).toZone('UTC'),
                DateTime('1999/01/01 09:00').toZone('UTC'))

        def testExpires(self):
            from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
            self.request.form['expirationDate'] = '1999/01/01 09:00'
            view = PropertiesActionView(self.portal.page, self.request)
            view()
            self.assertEqual(
                DateTime(self.portal.page.ExpirationDate()).toZone('UTC'),
                DateTime('1999/01/01 09:00').toZone('UTC'))

        def testRights(self):
            from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
            self.request.form['copyright'] = 'foobar'
            view = PropertiesActionView(self.portal.page, self.request)
            view()
            self.assertEqual(self.portal.page.Rights(), 'foobar')

        def testContributors(self):
            from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
            self.request.form['contributors'] = self.userList
            view = PropertiesActionView(self.portal.page, self.request)
            view()
            self.assertEqual(self.portal.page.Contributors(), ('one', 'two'))

        def testCreators(self):
            from plone.app.content.browser.contents.properties import PropertiesActionView  # noqa
            self.request.form['creators'] = self.userList
            view = PropertiesActionView(self.portal.page, self.request)
            view()
            self.assertEqual(self.portal.page.Creators(), ('one', 'two'))


class WorkflowTest(BaseTest):

    layer = PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING

    def convertDateTimeToIndexRepr(self, date):
        t_tup = date.toZone('UTC').parts()
        yr = t_tup[0]
        mo = t_tup[1]
        dy = t_tup[2]
        hr = t_tup[3]
        mn = t_tup[4]

        return ((((yr * 12 + mo) * 31 + dy) * 24 + hr) * 60 + mn)

    def testStateChange(self):
        from plone.app.content.browser.contents.workflow import WorkflowActionView  # noqa
        self.request.form['transition'] = 'publish'
        default_effective = DateTime('1969/12/31 00:00:00 {0}'.format(
            DateTime().timezone()
        ))
        default_effective_index = self.convertDateTimeToIndexRepr(
            default_effective
        )
        pc = getToolByName(self.portal, "portal_catalog")
        # i need to call it, to populate catalog indexes
        pc()
        self.assertEquals(
            pc.uniqueValuesFor('effective'),
            (default_effective_index,))
        view = WorkflowActionView(self.portal.page, self.request)
        view()
        workflowTool = getToolByName(self.portal, "portal_workflow")
        self.assertEqual(
            workflowTool.getInfoFor(self.portal.page, 'review_state'),
            'published')
        # commit to update indexes in catalog
        commit()
        effective_index = self.convertDateTimeToIndexRepr(
            self.portal.page.effective_date
        )
        self.assertEquals(
            pc.uniqueValuesFor('effective'),
            (effective_index,))


class RenameTest(BaseTest):

    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def test_folder_rename_objects(self):
        from plone.app.content.browser.contents.rename import RenameActionView
        uid = IUUID(self.portal.page)
        self.portal.invokeFactory('Document', id='page2', title='2nd page')
        uid2 = IUUID(self.portal.page2)
        self.request.form.update({
            'UID_0': uid,
            'newid_0': 'I am UnSafe! ',
            'newtitle_0': 'New!',
            'UID_1': uid2,
            'newid_1': '. ,;new id : _! ',
            'newtitle_1': 'Newer!'
        })
        view = RenameActionView(self.portal, self.request)
        view()
        self.assertEqual(self.portal['i-am-unsafe'].title, "New!")
        self.assertEqual(self.portal['new-id-_'].title, "Newer!")

    def test_default_page_updated_on_rename_objects(self):
        from plone.app.content.browser.contents.rename import RenameActionView
        self.portal.setDefaultPage('page')
        uid = IUUID(self.portal.page)
        self.request.form.update({
            'UID_0': uid,
            'newid_0': 'page-renamed',
            'newtitle_0': 'Page'
        })
        view = RenameActionView(self.portal, self.request)
        view()
        self.assertEqual(self.portal.getDefaultPage(), 'page-renamed')


class ContextInfoTest(BaseTest):

    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def testStateChange(self):
        from plone.app.content.browser.contents import ContextInfo
        view = ContextInfo(self.portal.page, self.request)
        result = json.loads(view())
        self.assertEqual(result['object']['Title'], 'page')
        self.assertTrue(len(result['breadcrumbs']) > 0)


class CutCopyLockedTest(BaseTest):
    """in folder contents """

    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.invokeFactory('Document', id="page", title="page")
        self.portal.page.reindexObject()

        self.env = {'HTTP_ACCEPT_LANGUAGE': 'en', 'REQUEST_METHOD': 'POST'}
        self.request = makerequest(self.layer['app']).REQUEST
        self.request.environ.update(self.env)
        self.request.form = {
            'selection': '["' + IUUID(self.portal.page) + '"]',
            '_authenticator': createToken(),
            'folder': '/'
        }
        self.request.REQUEST_METHOD = 'POST'

    def test_cut_object_when_locked(self):
        from plone.app.content.browser.contents.cut import CutActionView
        lockable = IRefreshableLockable(self.portal.page)
        lockable.lock()
        view = CutActionView(self.portal, self.request)
        view()
        self.assertEqual(len(view.errors), 1)


class DeleteDXTest(BaseTest):
    """Verify delete behavior from the folder contents view"""

    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.invokeFactory('Document', id="page", title="page")
        self.portal.page.reindexObject()

        self.env = {'HTTP_ACCEPT_LANGUAGE': 'en', 'REQUEST_METHOD': 'POST'}
        self.request = makerequest(self.layer['app']).REQUEST
        self.request.environ.update(self.env)
        self.request.form = {
            'selection': '["' + IUUID(self.portal.page) + '"]',
            '_authenticator': createToken(),
            'folder': '/'
        }
        self.request.REQUEST_METHOD = 'POST'

    def make_request(self):
        request = makerequest(self.layer['app'], environ=self.env).REQUEST
        self.request.environ.update(self.env)
        request.REQUEST_METHOD = 'POST'
        return request

    def test_delete_object(self):
        from plone.app.content.browser.contents.delete import DeleteActionView
        page_id = self.portal.page.id
        self.assertTrue(page_id in self.portal)
        view = DeleteActionView(self.portal, self.request)
        view()
        self.assertTrue(page_id not in self.portal)

    def test_delete_object_when_locked(self):
        from plone.app.content.browser.contents.delete import DeleteActionView
        lockable = IRefreshableLockable(self.portal.page)
        lockable.lock()
        view = DeleteActionView(self.portal, self.request)
        view()
        self.assertEqual(len(view.errors), 1)

    def test_delete_wrong_object_by_acquisition(self):
        page_id = self.portal.page.id
        f1 = self.portal.invokeFactory('Folder', id="f1", title="folder one")
        # created a nested page with the same id as the one at the site root
        p1 = self.portal[f1].invokeFactory(
            'Document',
            id=page_id,
            title="page"
        )
        self.assertEqual(p1, page_id)
        request2 = self.make_request()

        # both pages exist before we delete on
        for location in [self.portal, self.portal[f1]]:
            self.assertTrue(p1 in location)

        # instantiate two different views and delete the same object with each
        from plone.app.content.browser.contents.delete import DeleteActionView
        object_uuid = IUUID(self.portal[f1][p1])
        for req in [self.request, request2]:
            req.form = {
                'selection': '["{}"]'.format(object_uuid),
                '_authenticator': createToken(),
                'folder': '/{}/'.format(f1)
            }
            view = DeleteActionView(self.portal, req)
            view()

        # the root page exists, the nested one is gone
        self.assertTrue(p1 in self.portal)
        self.assertFalse(p1 in self.portal[f1])


if HAS_AT:
    class DeleteATTest(DeleteDXTest):

        layer = PLONE_APP_CONTENT_AT_INTEGRATION_TESTING


class RearrangeDXTest(BaseTest):
    """Verify rearrange feature from the folder contents view"""

    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.invokeFactory(
            'Folder',
            id="basefolder",
            title="Folder Base"
        )
        self.bf = self.portal.basefolder
        self.bf.reindexObject()
        for idx in range(0, 5):
            newid = "f{0:}".format(idx)
            self.bf.invokeFactory(
                'Folder',
                id=newid,
                # title in reverse order
                title="Folder {0:}".format(4-idx)
            )
            self.bf[newid].reindexObject()

        # create 3 documents in plone root
        for idx in range(0, 3):
            _id = "page_{}".format(idx)
            self.portal.invokeFactory(
                'Document',
                id=_id,
                title="Page {}".format(idx)
            )
            self.portal[_id].reindexObject()

        self.env = {'HTTP_ACCEPT_LANGUAGE': 'en', 'REQUEST_METHOD': 'POST'}
        self.request = makerequest(self.layer['app']).REQUEST
        self.request.environ.update(self.env)
        self.request.form = {
            'selection': '["' + IUUID(self.bf) + '"]',
            '_authenticator': createToken(),
            'folder': '/basefolder'
        }
        self.request.REQUEST_METHOD = 'POST'

    def test_initial_order(self):
        # just to be sure preconditions are fine
        #
        # initial ids are forward
        # and titles are set reversed!
        self.assertEqual(
            [(c[0], c[1].Title()) for c in self.bf.contentItems()],
            [
                ('f0', 'Folder 4'),
                ('f1', 'Folder 3'),
                ('f2', 'Folder 2'),
                ('f3', 'Folder 1'),
                ('f4', 'Folder 0'),
            ]
        )

    def test_rearrange_by_title(self):
        from plone.app.content.browser.contents.rearrange import RearrangeActionView  # noqa
        self.request.form.update({
            'rearrange_on': 'sortable_title',
        })
        view = RearrangeActionView(self.bf, self.request)
        view()
        self.assertEqual(
            [(c[0], c[1].Title()) for c in self.bf.contentItems()],
            [
                ('f4', 'Folder 0'),
                ('f3', 'Folder 1'),
                ('f2', 'Folder 2'),
                ('f1', 'Folder 3'),
                ('f0', 'Folder 4'),
            ],
        )

    def test_item_order_move_to_top(self):
        from plone.app.content.browser.contents.rearrange import ItemOrderActionView  # noqa
        self.request.form.update({
            'id': 'f2',
            'delta': 'top',
        })
        view = ItemOrderActionView(self.bf, self.request)
        view()
        self.assertEqual(
            [(c[0], c[1].Title()) for c in self.bf.contentItems()],
            [
                ('f2', 'Folder 2'),
                ('f0', 'Folder 4'),
                ('f1', 'Folder 3'),
                ('f3', 'Folder 1'),
                ('f4', 'Folder 0'),
            ]
        )

    def test_item_order_move_to_bottom(self):
        from plone.app.content.browser.contents.rearrange import ItemOrderActionView  # noqa
        self.request.form.update({
            'id': 'f2',
            'delta': 'bottom',
        })
        view = ItemOrderActionView(self.bf, self.request)
        view()
        self.assertEqual(
            [(c[0], c[1].Title()) for c in self.bf.contentItems()],
            [
                ('f0', 'Folder 4'),
                ('f1', 'Folder 3'),
                ('f3', 'Folder 1'),
                ('f4', 'Folder 0'),
                ('f2', 'Folder 2'),
            ]
        )

    def test_item_order_move_by_delta(self):
        from plone.app.content.browser.contents.rearrange import ItemOrderActionView  # noqa
        self.request.form.update({
            'id': 'f2',
            'delta': '-1',
        })
        view = ItemOrderActionView(self.bf, self.request)
        view()
        self.assertEqual(
            [(c[0], c[1].Title()) for c in self.bf.contentItems()],
            [
                ('f0', 'Folder 4'),
                ('f2', 'Folder 2'),
                ('f1', 'Folder 3'),
                ('f3', 'Folder 1'),
                ('f4', 'Folder 0'),
            ]
        )

    def test_item_order_move_by_delta_in_plone_root(self):
        from plone.app.content.browser.contents.rearrange import ItemOrderActionView  # noqa

        # first move the 'basefolder' to the top
        self.request.form.update({
            'id': 'basefolder',
            'delta': 'top',
        })
        view = ItemOrderActionView(self.portal, self.request)
        view()

        # move 'basefolder' two positions down
        self.request.form.update({
            'id': 'basefolder',
            'delta': '2',
            'subsetIds': '["basefolder", "page_0", "page_1", "page_2"]',
        })
        view = ItemOrderActionView(self.portal, self.request)
        view()

        self.assertEqual(
            [(c[0], c[1].Title()) for c in self.portal.contentItems()],
            [
                ('page_0', 'Page 0'),
                ('page_1', 'Page 1'),
                ('basefolder', 'Folder Base'),
                ('page_2', 'Page 2'),
            ]
        )

if HAS_AT:
    class RearrangeATTest(RearrangeDXTest):

        layer = PLONE_APP_CONTENT_AT_INTEGRATION_TESTING


class FolderFactoriesTest(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_folder_factories_regression(self):
        from plone.app.content.browser.folderfactories import (
            FolderFactoriesView as FFV)
        view = FFV(self.portal, self.request)
        self.request.form.update({
            'form.button.Add': 'yes',
            'url': self.portal.absolute_url()
        })
        view()
        self.assertEqual(self.request.response.headers.get('location'),
                         self.portal.absolute_url())

    def test_folder_factories(self):
        from plone.app.content.browser.folderfactories import (
            FolderFactoriesView as FFV)
        view = FFV(self.portal, self.request)
        self.request.form.update({
            'form.button.Add': 'yes',
            'url': 'http://www.foobar.com'
        })
        view()
        self.assertNotEqual(self.request.response.headers.get('location'),
                            'http://www.foobar.com')



# -*- coding: utf-8 -*-
from OFS.interfaces import IOrderedContainer
from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.utils import json_loads
from plone.folder.interfaces import IExplicitOrdering
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.interfaces import IPloneSiteRoot


class OrderContentsBaseAction(ContentsBaseAction):

    def getOrdering(self):
        if IPloneSiteRoot.providedBy(self.context):
            return self.context
        try:
            if self.context.aq_base.getOrdering():
                ordering = self.context.getOrdering()
            else:
                return None
        except AttributeError:
            if IOrderedContainer.providedBy(self.context):
                # Archetype
                return IOrderedContainer(self.context)
            return None
        if not IExplicitOrdering.providedBy(ordering):
            return None
        return ordering


class ItemOrderActionView(OrderContentsBaseAction):
    success_msg = _('Successfully moved item')
    failure_msg = _('Error moving item')

    def __call__(self):
        self.errors = []
        self.protect()
        id = self.request.form.get('id')
        ordering = self.getOrdering()

        if ordering is None:
            self.errors.append(_('This folder does not support ordering'))
            return self.message()

        delta = self.request.form['delta']

        if delta == 'top':
            ordering.moveObjectsToTop([id])
            return self.message()

        if delta == 'bottom':
            ordering.moveObjectsToBottom([id])
            return self.message()

        delta = int(delta)
        subset_ids = json_loads(self.request.form.get('subsetIds', 'null'))
        if subset_ids:
            position_id = [
                (ordering.getObjectPosition(i), i) for i in subset_ids
            ]
            position_id.sort()
            if subset_ids != [i for position, i in position_id]:
                self.errors.append(_('Client/server ordering mismatch'))
                return self.message()

        ordering.moveObjectsByDelta([id], delta, subset_ids)
        return self.message()


class RearrangeActionView(OrderContentsBaseAction):
    success_msg = _('Successfully rearranged folder')
    failure_msg = _(u'Can not rearrange folder')

    def __call__(self):
        self.protect()
        self.errors = []
        ordering = self.getOrdering()
        if ordering:
            catalog = getToolByName(self.context, 'portal_catalog')
            query = {
                'path': {
                    'query': '/'.join(self.context.getPhysicalPath()),
                    'depth': 1
                },
                'sort_on': self.request.form.get('rearrange_on'),
                'show_inactive': True
            }
            brains = catalog(**query)
            if self.request.form.get('reversed') == 'true':
                brains = [b for b in reversed(brains)]
            for idx, brain in enumerate(brains):
                ordering.moveObjectToPosition(brain.id, idx)
        else:
            self.errors.append(_(u'Not explicit orderable'))
        return self.message()



# -*- coding: utf-8 -*-
from datetime import datetime
from datetime import timedelta
from plone.app.content.testing import PLONE_APP_CONTENT_DX_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.dexterity.fti import DexterityFTI
from plone.protect.authenticator import createToken
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from zope.component import getMultiAdapter
from zope.component import getUtility

import json
import mock
import unittest


class ContentsCopyTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        # TYPE 1
        type1_fti = DexterityFTI('type1')
        type1_fti.klass = 'plone.dexterity.content.Container'
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = ['type1']
        type1_fti.behaviors = (
            'Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes',  # noqa
            'plone.app.dexterity.behaviors.metadata.IBasic'
        )
        self.portal.portal_types._setObject('type1', type1_fti)
        self.type1_fti = type1_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_keep_selection_order(self):
        """Keep the order of items the same as they were selected.
        """
        self.portal.invokeFactory('type1', id='f1', title='Folder 1')
        f1 = self.portal.f1
        f1.invokeFactory('type1', id='it1', title='Item 1')
        f1.invokeFactory('type1', id='it2', title='Item 2')
        f1.invokeFactory('type1', id='it3', title='Item 3')

        def _test_order(sel):
            self.request.form['selection'] = json.dumps([
                IUUID(f1[id_])
                for id_
                in sel
            ])
            view = f1.restrictedTraverse('@@fc-copy')
            view()
            self.assertEqual(
                [ob.id for ob in view.oblist],
                sel
            )

        _test_order(['it1', 'it2', 'it3'])
        _test_order(['it3', 'it1', 'it2'])


class ContentsDeleteTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        # TYPE 1
        type1_fti = DexterityFTI('type1')
        type1_fti.klass = 'plone.dexterity.content.Container'
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = ['type1']
        type1_fti.behaviors = (
            'Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes',  # noqa
            'plone.app.dexterity.behaviors.metadata.IBasic'
        )
        self.portal.portal_types._setObject('type1', type1_fti)
        self.type1_fti = type1_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_delete_success_with_private_anchestor(self):
        """Delete content item from a folder with private anchestor
        """
        # Create test content /it1/it2/it3
        self.portal.invokeFactory('type1', id='it1', title='Item 1')
        self.portal.it1.invokeFactory('type1', id='it2', title='Item 2')
        self.portal.it1.it2.invokeFactory('type1', id='it3', title='Item 3')
        self.assertEqual(len(self.portal.it1.it2.contentIds()), 1)

        # Block user access to it1m but leave access to its children
        self.portal.it1.__ac_local_roles_block__ = True
        del self.portal.it1.__ac_local_roles__[TEST_USER_ID]
        self.portal.it1.reindexObjectSecurity()
        self.portal.it1.it2.reindexObjectSecurity()

        # Remove test user global roles (leaving only local owner roles on it2)
        setRoles(self.portal, TEST_USER_ID, [])

        # Execute delete request
        selection = [self.portal.it1.it2.it3.UID()]
        self.request.form['folder'] = '/it1/it2'
        self.request.form['selection'] = json.dumps(selection)
        res = self.portal.it1.it2.restrictedTraverse('@@fc-delete')()

        # Check for successful deletion
        res = json.loads(res)
        self.assertEqual(res['status'], 'success')
        self.assertEqual(len(self.portal.it1.it2.contentIds()), 0)

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_delete_success_on_inactive_content(self):
        """Delete an expired content item from a folder.
        """
        # Create content
        self.portal.invokeFactory('type1', id='it1', title='Item 1')
        self.portal.it1.invokeFactory('type1', id='it2', title='Item 2')

        # Expire it2
        exp = datetime.now() - timedelta(days=10)
        self.portal.it1.it2.expiration_date = exp
        self.portal.it1.it2.reindexObject()

        # Remove test user global roles (leaving only local owner roles on it1
        # and below)
        setRoles(self.portal, TEST_USER_ID, [])

        # Execute delete request
        selection = [self.portal.it1.it2.UID()]
        self.request.form['folder'] = '/it1'
        self.request.form['selection'] = json.dumps(selection)
        res = self.portal.it1.restrictedTraverse('@@fc-delete')()

        # Check for successful deletion
        res = json.loads(res)
        self.assertEqual(res['status'], 'success')
        self.assertEqual(len(self.portal.it1.contentIds()), 0)


class ContentsPasteTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        # TYPE 1
        type1_fti = DexterityFTI('type1')
        type1_fti.klass = 'plone.dexterity.content.Container'
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = ['type1']
        type1_fti.behaviors = (
            'Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes',  # noqa
            'plone.app.dexterity.behaviors.metadata.IBasic'
        )
        self.portal.portal_types._setObject('type1', type1_fti)
        self.type1_fti = type1_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.invokeFactory('type1', id='it1', title='Item 1')

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_paste_success(self):
        """Copy content item and paste in portal root.
        """
        # # setup copying via @@fc-copy
        # from plone.uuid.interfaces import IUUID
        # self.request['selection'] = [IUUID(self.portal.it1)]
        # self.portal.restrictedTraverse('@@fc-copy')()

        self.request['__cp'] = self.portal.manage_copyObjects(['it1'])
        self.request.form['folder'] = '/'
        res = self.portal.restrictedTraverse('@@fc-paste')()

        res = json.loads(res)
        self.assertEqual(res['status'], 'success')
        self.assertEqual(len(self.portal.contentIds()), 2)

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_paste_success_paste_in_itself(self):
        """Copy content item and paste in itself. Because we can.
        """
        self.request['__cp'] = self.portal.manage_copyObjects(['it1'])
        self.request.form['folder'] = '/it1'
        res = self.portal.it1.restrictedTraverse('@@fc-paste')()

        res = json.loads(res)
        self.assertEqual(res['status'], 'success')
        self.assertEqual(len(self.portal.it1.contentIds()), 1)

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_paste_fail_constraint(self):
        """Fail pasting content item in itself when folder constraints don't
        allow to.
        """
        self.type1_fti.allowed_content_types = []  # set folder constraints
        self.request['__cp'] = self.portal.manage_copyObjects(['it1'])
        self.request.form['folder'] = '/it1'
        res = self.portal.it1.restrictedTraverse('@@fc-paste')()

        res = json.loads(res)
        self.assertEqual(res['status'], 'warning')
        self.assertEqual(len(self.portal.it1.contentIds()), 0)

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_paste_success_with_private_anchestor(self):
        """Copy content item and paste into a folder with private anchestor
        """
        # Create test content /it2/it3
        self.portal.invokeFactory('type1', id='it2', title='Item 2')
        self.portal.it2.invokeFactory('type1', id='it3', title='Item 3')
        self.assertEqual(len(self.portal.it2.it3.contentIds()), 0)

        # Block user access to it2, but leave access to its children
        self.portal.it2.__ac_local_roles_block__ = True
        del self.portal.it2.__ac_local_roles__[TEST_USER_ID]
        self.portal.it2.reindexObjectSecurity()
        self.portal.it2.it3.reindexObjectSecurity()

        # Remove test user global roles (leaving only local owner roles on it2)
        setRoles(self.portal, TEST_USER_ID, [])

        # Execute paste
        self.request['__cp'] = self.portal.manage_copyObjects(['it1'])
        self.request.form['folder'] = '/it2/it3'
        res = self.portal.it2.it3.restrictedTraverse('@@fc-paste')()

        # Check for successful paste
        res = json.loads(res)
        self.assertEqual(res['status'], 'success')
        self.assertEqual(len(self.portal.it2.it3.contentIds()), 1)


class ContentsRenameTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        # TYPE 1
        type1_fti = DexterityFTI('type1')
        type1_fti.klass = 'plone.dexterity.content.Container'
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = ['type1']
        type1_fti.behaviors = (
            'Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes',  # noqa
            'plone.app.dexterity.behaviors.metadata.IBasic'
        )
        self.portal.portal_types._setObject('type1', type1_fti)
        self.type1_fti = type1_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_rename_success_with_private_anchestor(self):
        """Rename content item from a folder with private anchestor
        """
        # Create test content /it1/it2/it3
        self.portal.invokeFactory('type1', id='it1', title='Item 1')
        self.portal.it1.invokeFactory('type1', id='it2', title='Item 2')
        self.portal.it1.it2.invokeFactory('type1', id='it3', title='Item 3')
        self.assertEqual(len(self.portal.it1.it2.contentIds()), 1)

        # Block user access to it1m but leave access to its children
        self.portal.it1.__ac_local_roles_block__ = True
        del self.portal.it1.__ac_local_roles__[TEST_USER_ID]
        self.portal.it1.reindexObjectSecurity()
        self.portal.it1.it2.reindexObjectSecurity()

        # Remove test user global roles (leaving only local owner roles on it2)
        setRoles(self.portal, TEST_USER_ID, [])

        # Execute rename request
        self.request.form['UID_1'] = self.portal.it1.it2.it3.UID()
        self.request.form['newid_1'] = 'it3bak'
        self.request.form['newtitle_1'] = 'Item 3 BAK'
        res = self.portal.it1.it2.restrictedTraverse('@@fc-rename')()

        # Check for successful deletion
        res = json.loads(res)
        self.assertEqual(res['status'], 'success')
        self.assertEqual(self.portal.it1.it2.it3bak.id, 'it3bak')
        self.assertEqual(self.portal.it1.it2.it3bak.title, 'Item 3 BAK')

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_rename_success_on_inactive_content(self):
        """Rename an expired content item from a folder.
        """
        # Create content
        self.portal.invokeFactory('type1', id='it1', title='Item 1')
        self.portal.it1.invokeFactory('type1', id='it2', title='Item 2')

        # Expire it2
        exp = datetime.now() - timedelta(days=10)
        self.portal.it1.it2.expiration_date = exp
        self.portal.it1.it2.reindexObject()

        # Remove test user global roles (leaving only local owner roles on it1
        # and below)
        setRoles(self.portal, TEST_USER_ID, [])

        # Execute rename request
        self.request.form['UID_1'] = self.portal.it1.it2.UID()
        self.request.form['newid_1'] = 'it2bak'
        self.request.form['newtitle_1'] = 'Item 2 BAK'
        res = self.portal.it1.restrictedTraverse('@@fc-rename')()

        # Check for successful deletion
        res = json.loads(res)
        self.assertEqual(res['status'], 'success')
        self.assertEqual(self.portal.it1.it2bak.id, 'it2bak')
        self.assertEqual(self.portal.it1.it2bak.title, 'Item 2 BAK')


class AllowUploadViewTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        # TYPE 1
        type1_fti = DexterityFTI('type1')
        type1_fti.klass = 'plone.dexterity.content.Container'
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = []
        type1_fti.behaviors = (
            'plone.app.dexterity.behaviors.metadata.IBasic',
        )
        self.portal.portal_types._setObject('type1', type1_fti)
        self.type1_fti = type1_fti

        # TYPE 2
        type2_fti = DexterityFTI('type1')
        type2_fti.klass = 'plone.dexterity.content.Item'
        type2_fti.filter_content_types = True
        type2_fti.allowed_content_types = []
        type2_fti.behaviors = (
            'plone.app.dexterity.behaviors.metadata.IBasic',
        )
        self.portal.portal_types._setObject('type2', type2_fti)
        self.type2_fti = type2_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.invokeFactory('type1', id='it1', title='Item 1')
        self.portal.invokeFactory('type2', id='it2', title='Item 2')

    def test_allow_upload(self):
        """Test, if file or images are allowed in a container in different FTI
        configurations.
        """

        # Test non-container, none allowed
        allow_upload = self.portal.it2.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload['allowUpload'], False)
        self.assertEqual(allow_upload['allowImages'], False)
        self.assertEqual(allow_upload['allowFiles'], False)

        # Test none allowed
        self.type1_fti.allowed_content_types = []
        allow_upload = self.portal.it1.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload['allowUpload'], False)
        self.assertEqual(allow_upload['allowImages'], False)
        self.assertEqual(allow_upload['allowFiles'], False)

        # Test images allowed
        self.type1_fti.allowed_content_types = ['Image']
        allow_upload = self.portal.it1.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload['allowUpload'], True)
        self.assertEqual(allow_upload['allowImages'], True)
        self.assertEqual(allow_upload['allowFiles'], False)

        # Test files allowed
        self.type1_fti.allowed_content_types = ['File']
        allow_upload = self.portal.it1.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload['allowUpload'], True)
        self.assertEqual(allow_upload['allowImages'], False)
        self.assertEqual(allow_upload['allowFiles'], True)

        # Test images and files allowed
        self.type1_fti.allowed_content_types = ['Image', 'File']
        allow_upload = self.portal.it1.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload['allowUpload'], True)
        self.assertEqual(allow_upload['allowImages'], True)
        self.assertEqual(allow_upload['allowFiles'], True)

        # Test files allowed, path via request variable
        self.type1_fti.allowed_content_types = ['File']
        # First, test on Portal root to see the difference
        allow_upload = self.portal.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())
        self.assertEqual(allow_upload['allowUpload'], True)
        self.assertEqual(allow_upload['allowImages'], True)
        self.assertEqual(allow_upload['allowFiles'], True)
        # Then, with path set to sub item
        allow_upload = self.portal.restrictedTraverse('@@allow_upload')
        allow_upload.request.form['path'] = '/plone/it1'
        allow_upload = json.loads(allow_upload())
        self.assertEqual(allow_upload['allowUpload'], True)
        self.assertEqual(allow_upload['allowImages'], False)
        self.assertEqual(allow_upload['allowFiles'], True)


class FCPropertiesTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        # Disable plone.protect for these tests
        self.request.environ['REQUEST_METHOD'] = 'POST'
        self.request.form['_authenticator'] = createToken()
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        # set available languages
        registry = getUtility(IRegistry)
        registry['plone.available_languages'] = ['en', 'de']

        self.portal.invokeFactory('Folder', 'main1')
        self.portal.main1.invokeFactory('Folder', 'sub1')
        self.portal.main1.sub1.invokeFactory('Folder', 'subsub1')
        self.portal.main1.invokeFactory('Document', 'sub2')
        self.portal.invokeFactory('Document', 'main2')

        self.setup_initial()

    def setup_initial(self):
        # Initial Settings
        self.portal.main1.exclude_from_nav = True
        self.portal.main1.sub1.exclude_from_nav = True
        self.portal.main1.sub1.subsub1.exclude_from_nav = True
        self.portal.main1.sub2.exclude_from_nav = True
        self.portal.main2.exclude_from_nav = True

        self.portal.main1.language = 'en'
        self.portal.main1.sub1.language = 'en'
        self.portal.main1.sub1.subsub1.language = 'en'
        self.portal.main1.sub2.language = 'en'
        self.portal.main2.language = 'en'

    def test_fc_properties__changes__no_recurse(self):
        """Test changing properties without recursion.
        """
        req = self.request
        req.form['language'] = 'de'
        req.form['exclude-from-nav'] = 'no'
        req.form['selection'] = '["{0}", "{1}"]'.format(
            IUUID(self.portal.main1),
            IUUID(self.portal.main2)
        )

        view = getMultiAdapter((self.portal, req), name=u'fc-properties')

        # Call the view and execute the actions
        view()

        self.assertEqual(self.portal.main1.language, 'de')
        self.assertEqual(self.portal.main2.language, 'de')
        self.assertEqual(self.portal.main1.sub1.language, 'en')
        self.assertEqual(self.portal.main1.sub1.subsub1.language, 'en')
        self.assertEqual(self.portal.main1.sub2.language, 'en')

        self.assertEqual(self.portal.main1.exclude_from_nav, False)
        self.assertEqual(self.portal.main2.exclude_from_nav, False)
        self.assertEqual(self.portal.main1.sub1.exclude_from_nav, True)
        self.assertEqual(self.portal.main1.sub1.subsub1.exclude_from_nav, True)
        self.assertEqual(self.portal.main1.sub2.exclude_from_nav, True)

    def test_fc_properties__changes__with_recurse(self):
        """Test changing properties without recursion.
        """
        req = self.request
        req.form['language'] = 'de'
        req.form['exclude-from-nav'] = 'no'
        req.form['recurse'] = 'yes'
        req.form['selection'] = '["{0}", "{1}"]'.format(
            IUUID(self.portal.main1),
            IUUID(self.portal.main2)
        )

        view = getMultiAdapter((self.portal, req), name=u'fc-properties')

        # Call the view and execute the actions
        view()

        self.assertEqual(self.portal.main1.language, 'de')
        self.assertEqual(self.portal.main2.language, 'de')
        self.assertEqual(self.portal.main1.sub1.language, 'de')
        self.assertEqual(self.portal.main1.sub1.subsub1.language, 'de')
        self.assertEqual(self.portal.main1.sub2.language, 'de')

        self.assertEqual(self.portal.main1.exclude_from_nav, False)
        self.assertEqual(self.portal.main2.exclude_from_nav, False)
        self.assertEqual(self.portal.main1.sub1.exclude_from_nav, False)
        self.assertEqual(self.portal.main1.sub1.subsub1.exclude_from_nav, False)  # noqa
        self.assertEqual(self.portal.main1.sub2.exclude_from_nav, False)



# -*- coding: utf-8 -*-
from zope.component.testing import setUp
from zope.component.testing import tearDown
import doctest
import unittest


def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('table.txt',
                             package='plone.app.content.browser',
                             optionflags=doctest.ELLIPSIS,
                             setUp=setUp, tearDown=tearDown)))



# -*- coding: utf-8 -*-
from plone.app.content.testing import PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING
from plone.app.content.testing import optionflags
from plone.testing import layered
import doctest
import unittest


doctests = (
    'basecontent.rst',
)


def test_suite():
    suite = unittest.TestSuite()
    tests = [
        layered(
            doctest.DocFileSuite(
                test_file,
                package='plone.app.content',
                optionflags=optionflags,
            ),
            layer=PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING,
        )
        for test_file in doctests
    ]
    suite.addTests(tests)
    return suite



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from plone.app.content.testing import PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING
from plone.testing.z2 import Browser
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import transaction
import unittest


class ReviewListTestCase(unittest.TestCase):
    """dsfsdaf"""

    layer = PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.uf = self.portal.acl_users
        self.uf.userFolderAddUser('reviewer', 'secret', ['Reviewer'], [])
        transaction.commit()
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = True
        self.wftool = getToolByName(self.portal, 'portal_workflow')

    def createDocument(self, id, title, description):
        setRoles(self.portal, TEST_USER_ID, ['Manager', ])
        self.portal.invokeFactory(id=id, type_name='Document')
        doc = getattr(self.portal, id)
        doc.title = title
        doc.description = description
        # we don't want it in the navigation
        doc.exclude_from_nav = True
        doc.reindexObject()
        transaction.commit()
        return doc

    def submitToReview(self, obj):
        '''call the workflow action 'submit' for an object'''
        self.wftool.doActionFor(obj, 'submit')

    def test_unauthenticated(self):
        '''
        unauthenticated users do not have the necessary permissions to view
        the review list
        '''
        self.browser.open('http://nohost/plone/full_review_list')
        self.assertTrue('Login Name' in self.browser.contents)

    def test_authenticated(self):
        '''
        authenticated users do have the necessary permissions to view
        the review list
        '''
        self.browser.addHeader('Authorization',
                               'Basic %s:%s' % ('reviewer', 'secret'))
        self.browser.open('http://nohost/plone/full_review_list')
        self.assertTrue('Full review list:' in self.browser.contents)

    def test_with_content(self):
        '''
        authenticated users do have the necessary permissions to view
        the review list
        '''
        doc = self.createDocument(
            'testdoc', 'Test Document', 'Test Description')
        self.wftool.doActionFor(doc, 'submit')
        transaction.commit()

        self.browser.addHeader('Authorization',
                               'Basic %s:%s' % ('reviewer', 'secret'))
        self.browser.open('http://nohost/plone/full_review_list')
        self.assertTrue('Full review list:' in self.browser.contents)
        # test if the table with review items contains an entry for testdoc
        self.assertTrue('value="/plone/testdoc"' in self.browser.contents)



# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from plone.memoize import ram
from zope.component import queryUtility
from zope.i18n.interfaces import ITranslationDomain
import json


def _cache_key(method, self, domain, language):
    return (domain, language,)


class i18njs(BrowserView):

    @ram.cache(_cache_key)
    def _gettext_catalog(self, domain, language):
        td = queryUtility(ITranslationDomain, domain)
        if td is None:
            return
        if language not in td._catalogs:
            baselanguage = language.split('-')[0]
            if baselanguage not in td._catalogs:
                return
            else:
                language = baselanguage
        _catalog = {}
        for mo_path in reversed(td._catalogs[language]):
            catalog = td._data[mo_path]._catalog
            if catalog is None:
                td._data[mo_path].reload()
                catalog = td._data[mo_path]._catalog
            _catalog.update(catalog._catalog)
        return _catalog

    def __call__(self, domain=None, language=None):

        if domain is None:
            catalog = {}
        else:
            if language is None:
                language = self.request['LANGUAGE']
            catalog = self._gettext_catalog(domain, language)

        response = self.request.response
        response.setHeader('Content-Type', 'application/json; charset=utf-8')
        response.setBody(json.dumps(catalog))
        return response



# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.interfaces import IStructureAction
from Products.CMFCore.interfaces._content import IFolderish
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ZODB.POSException import ConflictError
from zope.i18n import translate
from zope.interface import implementer


@implementer(IStructureAction)
class WorkflowAction(object):

    template = ViewPageTemplateFile('templates/workflow.pt')
    order = 7

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_options(self):
        return {
            'tooltip': translate(_('State'), context=self.request),
            'id': 'workflow',
            'icon': 'lock',
            'url': self.context.absolute_url() + '/@@fc-workflow',
            'form': {
                'title': _('Change workflow of selected items'),
                'template': self.template(),
                'dataUrl': self.context.absolute_url() + '/@@fc-workflow'
            }
        }


class WorkflowActionView(ContentsBaseAction):
    required_obj_permission = 'Modify portal content'
    success_msg = _('Successfully modified items')
    failure_msg = _('Failed to modify items')

    def __call__(self):
        self.pworkflow = getToolByName(self.context, 'portal_workflow')
        self.putils = getToolByName(self.context, 'plone_utils')
        self.transition_id = self.request.form.get('transition', None)
        self.comments = self.request.form.get('comments', '')
        self.recurse = self.request.form.get('recurse', 'no') == 'yes'
        if self.request.form.get('render') == 'yes':
            # asking for render information
            selection = self.get_selection()
            catalog = getToolByName(self.context, 'portal_catalog')
            brains = catalog(UID=selection, show_inactive=True)
            transitions = []
            for brain in brains:
                obj = brain.getObject()
                for transition in self.pworkflow.getTransitionsFor(obj):
                    tdata = {
                        'id': transition['id'],
                        'title': self.context.translate(transition['name'])
                    }
                    if tdata not in transitions:
                        transitions.append(tdata)
            return self.json({
                'transitions': transitions
            })
        else:
            return super(WorkflowActionView, self).__call__()

    def action(self, obj, bypass_recurse=False):
        transitions = self.pworkflow.getTransitionsFor(obj)
        if self.transition_id in [t['id'] for t in transitions]:
            try:
                # set effective date if not already set
                if obj.EffectiveDate() == 'None':
                    obj.setEffectiveDate(DateTime())

                self.pworkflow.doActionFor(obj, self.transition_id,
                                           comment=self.comments)
                if self.putils.isDefaultPage(obj):
                    self.action(obj.aq_parent, bypass_recurse=True)
                recurse = self.recurse and not bypass_recurse
                if recurse and IFolderish.providedBy(obj):
                    for sub in obj.values():
                        self.action(sub)
                obj.reindexObject()
            except ConflictError:
                raise
            except Exception:
                self.errors.append(
                    _('Could not transition: ${title}',
                      mapping={'title': self.objectTitle(obj)}))



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from OFS.interfaces import IFolder
from plone.app.dexterity.interfaces import IDXFileFactory
from plone.dexterity.interfaces import IDexterityFTI
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.permissions import AddPortalContent
from Products.Five.browser import BrowserView

import json
import logging
import mimetypes
import os


logger = logging.getLogger('plone')


def _bool(val):
    if val.lower() in ('t', 'true', '1', 'on'):
        return True
    return False


def _tus_int(val):
    try:
        return int(val)
    except:
        return 60 * 60  # default here...


possible_tus_options = {
    'tmp_file_dir': str,
    'send_file': _bool,
    'upload_valid_duration': _tus_int
}

TUS_ENABLED = False
if os.environ.get('TUS_ENABLED'):
    # tus resumable upload standard, see http://tus.io
    try:
        from tus import Tus, Zope2RequestAdapter
        tus_settings = {}
        for option, converter in possible_tus_options.items():
            name = 'TUS_%s' % option.upper()
            if name in os.environ:
                tus_settings[option] = converter(os.environ[name])

            tmp_file_dir = tus_settings.get('tmp_file_dir')
            if tmp_file_dir is None:
                logger.warn('You are trying to enable tus but no'
                            'TUS_TMP_FILE_DIR environment setting is set.')
            elif not os.path.exists(tmp_file_dir) or \
                    not os.path.isdir(tmp_file_dir):
                logger.warn('The TUS_TMP_FILE_DIR does not point to a valid '
                            'directory.')
            elif not os.access(tmp_file_dir, os.W_OK):
                logger.warn('The TUS_TMP_FILE_DIR is not writable')
            else:
                TUS_ENABLED = True
                logger.info('tus file upload support is successfully '
                            'configured')
    except ImportError:
        logger.warn('TUS_ENABLED is set; however, tus python package is '
                    'not installed')
else:
    try:
        import tus
        tus  # pyflakes
    except ImportError:
        pass
    else:
        logger.warn('You have the tus python package installed but it is '
                    'not configured for this plone client')


class FileUploadView(BrowserView):
    """
    Handle file uploads with potential
    special handling of TUS resumable uploads
    """

    tus_uid = None

    def __contains__(self, uid):
        return self.tus_uid and self.tus_uid == uid

    def __getitem__(self, uid):
        if self.tus_uid is None:
            self.tus_uid = uid
            self.__doc__ = 'foobar'  # why is this necessary?
            return self
        else:
            raise KeyError

    def __call__(self):
        # Check if user has permission to add content here
        sm = getSecurityManager()
        if not sm.checkPermission(AddPortalContent, self.context):
            response = self.request.RESPONSE
            response.setStatus(403)
            return "You are not authorized to add content to this folder."

        req = self.request
        tusrequest = False
        if TUS_ENABLED:
            adapter = Zope2RequestAdapter(req)
            tus = Tus(adapter, **tus_settings)
            if tus.valid:
                tusrequest = True
                tus.handle()
                if not tus.upload_finished:
                    return
                else:
                    filename = req.getHeader('FILENAME')
                    if tus.send_file:
                        filedata = req._file
                        filedata.filename = filename
                    else:
                        filepath = req._file.read()
                        filedata = open(filepath)
        if not tusrequest:
            if req.REQUEST_METHOD != 'POST':
                return
            filedata = self.request.form.get("file", None)
            if filedata is None:
                return
            filename = filedata.filename
        content_type = mimetypes.guess_type(filename)[0] or ""

        if not filedata:
            return

        ctr = getToolByName(self.context, 'content_type_registry')
        type_ = ctr.findTypeName(filename.lower(), content_type, '') or 'File'

        # Now check that the object is not restricted to be added in the
        # current context
        allowed_ids = [
            fti.getId() for fti in self.context.allowedContentTypes()
        ]
        if type_ not in allowed_ids:
            response = self.request.RESPONSE
            response.setStatus(403)
            if type_ == 'File':
                return "You cannot add a File to this folder, try another one"
            if type_ == 'Image':
                return (
                    "You cannot add an Image to this folder, "
                    "try another one"
                )

        # Determine if the default file/image types are DX or AT based
        dx_based = False
        pt = getToolByName(self.context, 'portal_types')
        if IDexterityFTI.providedBy(getattr(pt, type_)):
            factory = IDXFileFactory(self.context)
            dx_based = True
        else:
            from Products.ATContentTypes.interfaces import IATCTFileFactory
            factory = IATCTFileFactory(self.context)

        obj = factory(filename, content_type, filedata)

        result = {
            "type": '',
            "size": 0
        }

        if dx_based:
            if 'File' in obj.portal_type:
                result['size'] = obj.file.getSize()
                result['type'] = obj.file.contentType
            elif 'Image' in obj.portal_type:
                result['size'] = obj.image.getSize()
                result['type'] = obj.image.contentType
        else:
            result['type'] = obj.getContentType()
            try:
                result['size'] = obj.getSize()
            except AttributeError:
                result['size'] = obj.get_size()

        if tusrequest:
            tus.cleanup_file()
        result.update({
            'url': obj.absolute_url(),
            'name': obj.getId(),
            'UID': IUUID(obj),
            'filename': filename
        })

        self.request.response.setHeader(
            'Content-Type', 'application/json; charset=utf-8'
        )
        return json.dumps(result)


class AllowUploadView(BrowserView):

    def __call__(self):
        """Return JSON structure to indicate if File or Image uploads are
        allowed in the current container.
        """
        self.request.response.setHeader(
            'Content-Type', 'application/json; charset=utf-8'
        )
        context = self.context
        if self.request.form.get('path'):
            context = context.restrictedTraverse(self.request.form.get('path'))

        allow_images = False
        allow_files = False
        if IFolder.providedBy(context):
            allowed_types = [t.getId() for t in context.allowedContentTypes()]
            allow_images = u'Image' in allowed_types
            allow_files = u'File' in allowed_types

        return json.dumps({
            'allowUpload': allow_images or allow_files,
            'allowImages': allow_images,
            'allowFiles': allow_files
        })
